@@ -127,6 +127,7 @@ eNextState SMTPC_read_greeting(SmtpOutMsg *Msg)
 	/* Process the SMTP greeting from the server */
 	AsyncIO *IO = &Msg->IO;
 	SMTP_DBG_READ();
+	SetSMTPState(IO, eSTMPsmtp);
 
 	if (!SMTP_IS_STATE('2')) {
 		if (SMTP_IS_STATE('4'))
@@ -193,6 +194,11 @@ eNextState SMTPC_read_HELO_reply(SmtpOutMsg *Msg)
 		else
 			SMTP_VERROR(5);
 	}
+	if (Msg->pCurrRelay != NULL)
+	{
+		if (strstr(ChrPtr(Msg->IO.IOBuf), "LOGIN") != NULL)
+			Msg->SendLogin = 1;
+	}
 	if ((Msg->pCurrRelay == NULL) ||
 	    (Msg->pCurrRelay->User == NULL))
 		READ_NEXT_STATE(eFROM); /* Skip auth... */
@@ -417,11 +423,13 @@ eNextState SMTPC_read_DATAcmd_reply(SmtpOutMsg *Msg)
 	SMTP_DBG_READ();
 
 	if (!SMTP_IS_STATE('3')) {
+		SetSMTPState(IO, eSTMPfailOne);
 		if (SMTP_IS_STATE('4'))
 			SMTP_VERROR(3);
 		else
 			SMTP_VERROR(5);
 	}
+	SetSMTPState(IO, eSTMPsmtpdata);
 	return eSendReply;
 }
 
@@ -469,6 +477,7 @@ eNextState SMTPC_read_data_body_reply(SmtpOutMsg *Msg)
 			SMTP_VERROR(5);
 	}
 
+	SetSMTPState(IO, eSTMPsmtpdone);
 	/* We did it! */
 	StrBufPlain(Msg->MyQEntry->StatusMessage,
 		    &ChrPtr(Msg->IO.RecvBuf.Buf)[4],
