@@ -68,7 +68,6 @@ typedef struct _stmp_out_msg {
 	char node[1024];
 	char name[1024];
 	char mailfrom[1024];
-
 	long SendLogin;
 	long Flags;
 } SmtpOutMsg;
@@ -124,3 +123,22 @@ int smtp_resolve_recipients(SmtpOutMsg *SendMsg);
 #define SMTPCM_syslog(LEVEL, FORMAT)		\
 	DBGLOG(LEVEL) syslog(LEVEL,		\
 			     "SMTPCQ: " FORMAT)
+
+
+
+typedef enum __smtpstate {
+	eSTMPmxlookup,
+	eSTMPevaluatenext,
+	eSTMPalookup,
+	eSTMPaaaalookup,
+	eSTMPconnecting,
+	eSTMPsmtp,
+	eSTMPsmtpdata,
+	eSTMPsmtpdone,
+	eSTMPfinished,
+	eSTMPfailOne,
+	eSMTPFailTemporary,
+	eSMTPFailTotal
+} smtpstate;
+
+void SetSMTPState(AsyncIO *IO, smtpstate State);
